/*
 * File: cdlabel.cc
 * Code: Ben Walding
 * Date: November 1997
 * An audio CD insert creation utility.
 * Useful for creating labels for those albums you have which you have lost the cases for.
 * Also useful for creating cases for CDs which don't come with normal cases
 * Pearl Jam Vitalogy, Pantera (some strange round case) etc.
 *
 * Reads the workmandb, then modifies some xfig templates
 * and then prints these to postscript.
 * Protected by the GNU Public License v2.0  <distrib>/GPL.LICENSE
 */


/* Standard includes */
#include <stdio.h>  
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>

/* QT includes */
#include <qapp.h>
#include <qpushbt.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmsgbox.h>

/* Custom includes */
#include "cdoption.h"
#include "cdlabel.h"
#include "workmandb.h"
#include "cdlabel.moc"
#include "about.h"
#include "database.h"

#define INFO "CDLabel Pro " VERSION " conceived and written by Ben Walding\n" \
             "Email: e3322439@student.uq.edu.au" \
			 "See GPL.LICENSE or http://www.gnu.org for restrictions under GPL v2.0" \
			 "All I request is that you not release other versions of it without\n" \
			 "at least asking. E-mail me any comments or ideas\n"

// We need this out here, because we sometimes need to access the object from non-C++ functions
// This may be a bad way to do this... tell me more if you think so!
mainInterface *interface;
QApplication *app;


int main(int argc, char *argv[]) {
  QApplication a(argc, argv);
  interface = new mainInterface;

  puts(INFO);

  app = &a;

  read_rc();  //Load options from RC file
  a.setStyle(global_options.motifstyle?MotifStyle:WindowsStyle);


  if (global_options.splash) 
	interface->showSplash();
  
  interface->changedOptions();

  load_database();
  interface->setStatus("Database loaded");
  interface->update_album();/*YUCK - XXX XXX XXX */


  interface->setCaption("CD Label Pro v" VERSION);
  a.setMainWidget(interface);

  interface->setGeometry(100,100, 600, 600);
  interface->show();

  // There has to be a better way XXX XXX XXX
  interface->connect(interface->b_quit, SIGNAL(clicked()), &a, SLOT(quit()));

  return a.exec();
}

mainInterface::mainInterface(QWidget *parent, const char *name) : QWidget(parent, name)
{
  hide();
  splashScreen = NULL;
  //  hide();
  cd_list = new MyAlbumList(this, "cd_list");

  cd_list->update();

  QToolTip::add(cd_list, "List of currently available CD titles");
  
  //  printf("mainint: width() %d  height() %d\n", width(), height());
  cd_info = new cdInfo(this, "cd_info");
  cd_info->show();

  // Buttons on right of screen 
  b_display = new QPushButton("&Display", this, "b_display");
  QToolTip::add(b_display, "Display current view of CD label");
  b_print = new QPushButton("&Print label...", this, "b_print");
  QToolTip::add(b_print, "Print CD label using lpr to a printer (non-functional)");
  b_options = new QPushButton("&Options...", this, "b_options");
  QToolTip::add(b_options, "CDLabel options");
  b_quit = new QPushButton("&Quit", this, "b_quit");
  QToolTip::add(b_quit, "Quit CDLabel");
  b_about = new QPushButton("&About", this, "b_about");
  QToolTip::add(b_about, "About CDLabel");
  b_refresh = new QPushButton("&Refresh", this, "b_refresh");
  QToolTip::add(b_refresh, "Refresh the CD listing");

  l_status = new QLabel(this, "l_status");
  QToolTip::add(l_status, "Status / Information");
  l_status->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  l_status->show();
  
  b_print->setEnabled(FALSE);b_display->setEnabled(FALSE);

  b_print->show();
  b_display->show();
  b_quit->show();

  connect(cd_list, SIGNAL(selected(int)), SLOT(display_info(int)));
  connect(b_display,SIGNAL(clicked()), this, SLOT(displayLabel()));
  connect(b_print, SIGNAL(clicked()), this, SLOT(printLabel()));
  connect(b_options, SIGNAL(clicked()), this, SLOT(displayOptions()));
  connect(b_about, SIGNAL(clicked()), this, SLOT(showAbout()));
  connect(b_refresh, SIGNAL(clicked()), this, SLOT(refresh()));

}

// This is a mess!
// It should be made a little neater XXX XXX XXX
#define WIDTH_DIV 4
void mainInterface::resizeEvent(QResizeEvent *) 
{
  /* This is its own widget  - only use 3/5 of screen */
  cd_list->setGeometry(10, 10, width() * 3 / WIDTH_DIV  - 5, height()/2 - 5);
  cd_info->setGeometry(10, height() / 2 + 5, width() * 3 / WIDTH_DIV - 5,  height()/2 - 45);

  l_status->setGeometry(10, height() - 30, width() - 20, 20);

  b_display->setGeometry(width() * 3 / WIDTH_DIV + 15, 10, width() / WIDTH_DIV - 20, 20);
  b_print->setGeometry(width() * 3 / WIDTH_DIV + 15, 35, width() / WIDTH_DIV - 20, 20);  
  b_options->setGeometry(width() * 3 / WIDTH_DIV + 15, 60, width() / WIDTH_DIV - 20, 20);
  b_quit->setGeometry(width() * 3 / WIDTH_DIV + 15, 85, width() / WIDTH_DIV - 20, 20);
  b_about->setGeometry(width() * 3 / WIDTH_DIV + 15, 110, width() / WIDTH_DIV - 20, 20);

  b_refresh->setGeometry(width() * 2 / WIDTH_DIV, height() / 2 + 20, width() / WIDTH_DIV, 20);
}

int albumIndex = -1; // -1 == nothing selected

void mainInterface::display_info(int value)
{
  bool newStatus;
  if (value == -1) 
	newStatus = FALSE;
  else 
	newStatus = TRUE;

  b_print->setEnabled(newStatus);
  b_display->setEnabled(newStatus);

  cd_info->display_info(value);
}

// Nasty hack to avoid doing mainInterface->cd_list->update()
// Plus the main interface may want to know.
void mainInterface::update_album(void) 
{
  cd_list->update();
}
  

void mainInterface::genLabel() {
  static FILE *data;
  char filename[128];
  char cmdbuf[1024];
  char buf[1024];
  Album *album_ptr = getAlbumIndex(albumIndex);
  Track *track_ptr;

  if (albumIndex == -1) {
    QMessageBox::critical( this, "CDLabel",	"You have not selected a CD yet!");
    return;
  } 

  tmpnam(filename);
   
  if (!(data = fopen(filename, "w"))) {
    perror(filename);
    QMessageBox::information( this, "CDLabel",
			      "Unable to open temporary file");
    
    return;
  }
  
  fprintf(data, "%s\n%s\n", album_ptr->title(), album_ptr->artist());
  fprintf(data, "%2d:%02d\n", album_ptr->min(), album_ptr->sec());
  fprintf(data, "%d\n", album_ptr->ntracks);
  fprintf(data, "%s\n", global_options.cdlogo == TRUE ? global_options.cdlogo_path : "/dev/null");
 
  
  for (track_ptr = album_ptr->tracks; track_ptr; track_ptr = track_ptr->next)
    fprintf(data, "%s\n%2d:%02d\n", track_ptr->title(), track_ptr->min(), track_ptr->sec());
  
  fclose(data);
  
  sprintf(cmdbuf, "perl %s/cdlabel.pl %s %s %s %s", 
		  global_options.loc_template, filename, global_options.loc_template, 
		  "cd_front.fig", "cd_back.fig");
  system(cmdbuf);//Should check return levels XXX XXX XXX and then jump out if there is a failure

  unlink(filename);

  sprintf(cmdbuf, "fig2dev -L ps -P cd_front.fig cd_front.ps");
  system(cmdbuf);
  if (global_options.delete_intermediate) 
    unlink("cd_front.fig");
    
  sprintf(cmdbuf, "fig2dev -L ps -P cd_back.fig cd_back.ps");
  system(cmdbuf);
  if (global_options.delete_intermediate) 
    unlink("cd_back.fig");
  

  //b_display->setEnabled(TRUE);    // Can now display, because item is generated
  sprintf(buf, "\"%s\" has been generated", album_ptr->title());
  setStatus(buf);
  return;
}

void mainInterface::refresh(void) {
  clear_database();
  load_database();
  update_album();

  display_info(-1);
  setStatus("workman database has been reloaded");
}


void mainInterface::printLabel(void) {
  genLabel(); //Fix this -> add a variable to detect if current selection has been generated? XXX XXX XXX
  
  //User wants to print to lpr...
  if (global_options.out_lpr) {
	char cmdbuf[1024];
	char statbuf[1024];
	if (global_options.lpr_split) {
	  sprintf(cmdbuf, "%s %s &", global_options.lpr_default, "cd_front.fig");
	  sprintf(statbuf, "Executing: %s", cmdbuf);
	  setStatus(statbuf);
	  system(cmdbuf);

	  sprintf(cmdbuf, "%s %s &", global_options.lpr_default, "cd_back.fig");
	  sprintf(statbuf, "Executing: %s", cmdbuf);
	  setStatus(statbuf);
	  system(cmdbuf);
	} else {
	  sprintf(cmdbuf, "%s %s %s &", global_options.lpr_default, "cd_front.fig", "cd_back.fig");
	  sprintf(statbuf, "Executing: %s", cmdbuf);
	  setStatus(statbuf);
	  system(cmdbuf);
	}
  }
  //Currently gen label does the file generation automatically.
  //Ideally this should generate files in the correct place (current directory)
  //rather than temp ones which genLabel makes (use a copy?) -> leave until v2!
}
void mainInterface::displayLabel(void) {
  char buf[128];
  //XXX XXX XXX change to use options XXX XXX XXX this is broken
  //Check for existence of file XXX XXX XXX
  genLabel();

  sprintf(buf, "%s cd_back.ps &", global_options.ps_view);
  system(buf);

  sprintf(buf, "%s cd_front.ps &", global_options.ps_view);
  system(buf);
  sprintf(buf, "Please wait... %s called to view labels", global_options.ps_view);
  setStatus(buf);

}

void mainInterface::displayOptions() 
{
  cd_options = new cdOptions(this, "cd_options", TRUE, WStyle_DialogBorder);
}

/* If the options have changed, send signal to this slot - What is the current point in this? XXX XXX XXX*/
void mainInterface::changedOptions()
{
}


void mainInterface::changedSortOrder()
{ 
  char buf[1024];
  sort_database();
  update_album();
  cd_info->display_info(-1);

  sprintf(buf, "Database sorted by %s", global_options.sortartist ? "artist" : "title");
  setStatus(buf);
}

void mainInterface::showAbout() 
{
  showSplash();
}


void mainInterface::showSplash()
{
#ifdef SPLASH
  splashScreen = new Splash(NULL, "splash");
  splashScreen->show();
#endif
}


// I believe we have to destroy it to regain colors XXX XXX XXX
void mainInterface::hideSplash()
{
#ifdef SPLASH
  delete splashScreen;
  splashScreen = NULL;
#endif
}

static dbEntries = 0;

void mainInterface::dbUpdate(char *title) 
{
  char buf[128];
  dbEntries++;
  sprintf(buf, "%40s %5d", title, dbEntries);
  setStatus(buf);
}

void mainInterface::setStatus(char *status)
{
  l_status->setText(status);
}
